/* signal.c - signals handler
 *
 * Copyright (C) 2006-2007 Abilis
 * Copyright (C) 2006-2007 Antek srl
 *
 * Author: Elisa Manara <e@entropika.net>
 * Last update: Thu Nov 23 10:32:13 CET 2006
 *
 * This file is part of Abilis Collector for *nix.
 *
 * Abilis Collector is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Abilis Collector is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *                                   
 * You should have received a copy of the GNU General Public License along                        
 * with Abilis Collector; see the file COPYING.  If not, write to the Free                        
 * Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. 
 */  

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "define.h"

volatile sig_atomic_t end_signal = 0;
volatile sig_atomic_t status_signal = 0;
volatile sig_atomic_t reset_signal = 0;

int collectorStart(int argc, char **argv, struct CfgData *config, 
    struct CfgClient *client);
void collectorTerminate(struct CfgData *config, struct CfgClient *client);
void collectorStatus(struct CfgData *config, struct CfgClient *client);
int collectorSignalRequest(int argc, char **argv, pid_t c_pid);

int collectorHandleSignals(int argc, char **argv, struct CfgData *config, 
    struct CfgClient *client)
{
    if(end_signal) {
	printDebug("*** Exit requested: collector exiting now.\n");
	collectorTerminate(config, client);
	setShutdown(COLLECTOR_HALT, 0);
	end_signal = 0;
    }

    if(status_signal) {
	printDebug("*** Status requested: dump follows.\n");
	collectorStatus(config, client);
	status_signal = 0;
    }

    if(reset_signal) {
	printDebug("*** Reset requested: collector restarting now.\n");
	collectorTerminate(config, client);
	setShutdown(COLLECTOR_CONTINUE, 0);
	reset_signal = 0;
    }

    return 0;
}
	
void sigHandler(int signum)
{
    switch(signum) {
    case SIGINT:
    case SIGQUIT:
    case SIGILL:
    case SIGSEGV:
    case SIGPIPE:
    case SIGTERM:
	end_signal++;
	break;
    case SIGHUP:
	reset_signal++;
	break;
    case SIGUSR1:
	status_signal++;
	break;
    default:
	break;
    }
}

int collectorSignalRequest(int argc, char **argv, pid_t c_pid)
{
    if(argc > 1) { 
	if((strcmp(argv[1], "stop")) == 0) {
	    printDebug("Sending 'stop' request to running collector (pid=%i)\n",
		c_pid);
	    kill(c_pid, SIGINT);
	    return 1;
	} else if ((strcmp(argv[1], "reload")) == 0) {
	    printDebug("Sending 'reload' request to running collector (pid=%i)\n",
		c_pid);
	    kill(c_pid, SIGHUP);
	    return 1;
	} else if ((strcmp(argv[1], "status")) == 0) {
	    printDebug("Sending 'status' request to running collector (pid=%i)\n",
		c_pid);
	    kill(c_pid, SIGUSR1);
	    return 1;
	}
    }

    /* double instance - exit now */
    printError("Sorry, another collector (pid=%i) is running. Must exit.\n",
	c_pid);
    return (-1);
}

void collectorSignalsConnect(int *signals)
{
    int i;
    for(i = 0; signals[i]; i++)
	signal(signals[i],  sigHandler);
}

int collectorSignalsBlock(int *signals)
{
    int i;
    sigset_t set;

    sigemptyset(&set);
    for(i = 0; signals[i]; i++)
	sigaddset(&set, signals[i]);

    return sigprocmask(SIG_BLOCK, &set, NULL);
}

int collectorSignalsUnblock(int *signals)
{
    int i;
    sigset_t set;

    sigemptyset(&set);
    for(i = 0; signals[i]; i++)
	sigaddset(&set, signals[i]);

    return sigprocmask(SIG_UNBLOCK, &set, NULL);
}
/* 
 * vim: softtabstop=4 shiftwidth=4 
 */


